"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiEntityRef = void 0;
const _ = require("lodash");
const api_property_ref_decorator_1 = require("./api-property-ref.decorator");
const hasApiDecorators = (value) => _.isArray(value[api_property_ref_decorator_1.apiDecoratorsSymbol]);
function ApiEntityRef(EntityConstructor, { groups = [] } = {}) {
    return (_target) => {
        const target = _target;
        if (!_.isFunction(target)) {
            throw new Error(`${ApiEntityRef.name} decorator is applicable only to classes`);
        }
        target[api_property_ref_decorator_1.entityConstructorSymbol] = EntityConstructor;
        const targetAndAllParents = [];
        const PARENTS_LIMIT = 16;
        let i = 0;
        for (let targetOrItsParent = target; i < PARENTS_LIMIT && targetOrItsParent && targetOrItsParent !== Object.prototype; i++, targetOrItsParent = Object.getPrototypeOf(targetOrItsParent)) {
            if (!hasApiDecorators(targetOrItsParent))
                continue;
            targetAndAllParents.push(targetOrItsParent);
        }
        targetAndAllParents.reverse().forEach(targetOrItsParent => targetOrItsParent[api_property_ref_decorator_1.apiDecoratorsSymbol].forEach((v) => {
            v.swagger(EntityConstructor);
            v.validators(EntityConstructor, target, groups);
            v.forClassTransformations(EntityConstructor);
        }));
        if (i >= PARENTS_LIMIT) {
            console.warn(`Warning! ${ApiEntityRef.name}: parents limit faced for ${target.name}. It looks like a recursive prototype link`);
        }
    };
}
exports.ApiEntityRef = ApiEntityRef;
//# sourceMappingURL=api-entity-ref.decorator.js.map