"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiPropertyRef = exports.ApiPropertyRefDecorator = exports.entityConstructorSymbol = exports.apiDecoratorsSymbol = void 0;
const _ = require("lodash");
const constants_1 = require("@nestjs/swagger/dist/constants");
const api_property_decorator_1 = require("@nestjs/swagger/dist/decorators/api-property.decorator");
const class_transformer_1 = require("./@import-fix/class-transformer");
const class_validator_1 = require("./@import-fix/class-validator");
const utils_1 = require("./utils");
exports.apiDecoratorsSymbol = Symbol('api-decorators');
exports.entityConstructorSymbol = Symbol('entity-constructor');
const validatorsByConstructor = new Map();
class ApiPropertyRefDecorator {
    constructor(classProto, propertyKey, swaggerOptions, options) {
        this.swaggerOptions = swaggerOptions;
        this.classValidatorStorage = this.constructor.classValidatorStorage;
        this.copySwaggerDecorators = (EntityConstructor) => {
            const properties = Reflect.getMetadata(constants_1.DECORATORS.API_MODEL_PROPERTIES_ARRAY, this.classProto) || [];
            const key = `:${this.propertyKey}`;
            if (properties.includes(key))
                return;
            const existingEntityMetadata = Reflect.getMetadata(constants_1.DECORATORS.API_MODEL_PROPERTIES, EntityConstructor.prototype, this.normalizedEntityPropertyKey);
            if (existingEntityMetadata) {
                const targetMetadata = Reflect.getMetadata(constants_1.DECORATORS.API_MODEL_PROPERTIES, this.classProto, this.propertyKey);
                const existingMetadata = targetMetadata || existingEntityMetadata;
                const newMetadata = _.pickBy(this.swaggerOptions, _.negate(_.isUndefined));
                const metadataToSave = this.options.overrideExisting
                    ? {
                        ...existingMetadata,
                        ...newMetadata,
                    }
                    : {
                        ...newMetadata,
                        ...existingMetadata,
                    };
                Reflect.defineMetadata(constants_1.DECORATORS.API_MODEL_PROPERTIES, metadataToSave, this.classProto, this.propertyKey);
                Reflect.defineMetadata(constants_1.DECORATORS.API_MODEL_PROPERTIES_ARRAY, [...properties, `:${this.propertyKey}`], this.classProto);
            }
            else if (!_.isEmpty(this.swaggerOptions)) {
                (0, api_property_decorator_1.createApiPropertyDecorator)(this.swaggerOptions)(this.classProto, this.propertyKey);
            }
        };
        this.copyClassValidatorDecorators = (EntityConstructor, realTarget, groups = []) => {
            const groupsHashKey = [...groups].sort().join();
            let validationGroups = validatorsByConstructor.get(EntityConstructor);
            if (!validationGroups) {
                validationGroups = {};
                validatorsByConstructor.set(EntityConstructor, validationGroups);
            }
            let validators = validationGroups === null || validationGroups === void 0 ? void 0 : validationGroups[groupsHashKey];
            if (!validators) {
                validators = new Map();
                this.classValidatorStorage
                    .getTargetValidationMetadatas(EntityConstructor, undefined, undefined, undefined, groups)
                    .forEach((validator) => {
                    let propValidators = validators === null || validators === void 0 ? void 0 : validators.get(validator.propertyName);
                    if (!propValidators) {
                        propValidators = [];
                        validators === null || validators === void 0 ? void 0 : validators.set(validator.propertyName, propValidators);
                    }
                    propValidators.push(validator);
                });
                validationGroups[groupsHashKey] = validators;
            }
            const targetValidators = validators.get(this.normalizedEntityPropertyKey);
            if (targetValidators) {
                targetValidators.forEach((validator) => {
                    var _a, _b;
                    const groupsDiff = _.difference(validator.groups, groups);
                    const hasNoGroupsDiff = _.isEmpty(groupsDiff);
                    const updatedValidator = new class_validator_1.ValidationMetadata({
                        ...validator,
                        propertyName: this.propertyKey,
                        target: realTarget,
                        validationOptions: {
                            ...(validator.validationTypeOptions || {}),
                            always: hasNoGroupsDiff ? true : (_b = (_a = validator.validationTypeOptions) === null || _a === void 0 ? void 0 : _a.always) !== null && _b !== void 0 ? _b : undefined,
                            groups: hasNoGroupsDiff ? undefined : groupsDiff,
                        },
                    });
                    this.classValidatorStorage.addValidationMetadata(updatedValidator);
                });
            }
        };
        const apiPropertyRefClassName = ApiPropertyRef.name;
        if (!(0, utils_1.isClass)(classProto.constructor)) {
            throw new Error(`${apiPropertyRefClassName} decorator is applicable only to class properties`);
        }
        this.classProto = classProto;
        if (_.isSymbol(propertyKey)) {
            const entityNameInfo = ` Entity: ${String(this.classProto.constructor.name)}`;
            throw new Error(`${apiPropertyRefClassName} decorator is not applicable to 'symbol' properties.${entityNameInfo}`);
        }
        this.propertyKey = propertyKey;
        this.normalizedEntityPropertyKey = options.entityPropertyKey || propertyKey;
        this.options = {
            overrideExisting: true,
            ...options,
        };
    }
    addMetadata() {
        (this.classProto.constructor[exports.apiDecoratorsSymbol]
            = Object.prototype.hasOwnProperty.call(this.classProto.constructor, exports.apiDecoratorsSymbol)
                ? (this.classProto.constructor[exports.apiDecoratorsSymbol] || [])
                : []).push({
            swagger: this.copySwaggerDecorators,
            validators: this.copyClassValidatorDecorators,
            forClassTransformations: this.copyClassTransformerDecorators.bind(this),
        });
    }
    copyClassTransformerDecorators(EntityConstructor) {
        const storage = class_transformer_1.defaultMetadataStorage;
        const type = storage.findTypeMetadata(EntityConstructor, this.normalizedEntityPropertyKey);
        if (type) {
            const copy = { ...type, target: this.classProto.constructor, propertyName: this.propertyKey };
            storage.addTypeMetadata(copy);
        }
        const expose = storage.findExposeMetadata(EntityConstructor, this.normalizedEntityPropertyKey);
        if (expose) {
            const copy = { ...expose, target: this.classProto.constructor, propertyName: this.propertyKey };
            storage.addExposeMetadata(copy);
        }
        const exposeOnClass = storage.findExposeMetadata(EntityConstructor, undefined);
        if (exposeOnClass) {
            const copy = { ...exposeOnClass, target: this.classProto.constructor, propertyName: undefined };
            storage.addExposeMetadata(copy);
        }
        const exclude = storage.findExcludeMetadata(EntityConstructor, this.normalizedEntityPropertyKey);
        if (exclude) {
            const copy = { ...exclude, target: this.classProto.constructor, propertyName: this.propertyKey };
            storage.addExcludeMetadata(copy);
        }
        const excludeOnClass = storage.findExcludeMetadata(EntityConstructor, undefined);
        if (excludeOnClass) {
            const copy = { ...excludeOnClass, target: this.classProto.constructor, propertyName: undefined };
            storage.addExcludeMetadata(copy);
        }
        const transformMetadataCopyCb = (meta) => {
            const copy = { ...meta, target: this.classProto.constructor, propertyName: this.propertyKey };
            storage.addTransformMetadata(copy);
        };
        _.values(class_transformer_1.TransformationType).forEach(transformationTypeValue => storage.findTransformMetadatas(EntityConstructor, this.normalizedEntityPropertyKey, transformationTypeValue).forEach(transformMetadataCopyCb));
    }
}
exports.ApiPropertyRefDecorator = ApiPropertyRefDecorator;
ApiPropertyRefDecorator.classValidatorStorage = (0, class_validator_1.getMetadataStorage)();
function ApiPropertyRef(swaggerOptions = {}, options = {}) {
    return (target, propertyKey) => new ApiPropertyRefDecorator(target, propertyKey, swaggerOptions, options).addMetadata();
}
exports.ApiPropertyRef = ApiPropertyRef;
//# sourceMappingURL=api-property-ref.decorator.js.map